/*****************************************************************************\
 * Vvoj okennho systmu ve WinAPI
 * Bakalsk prce 2006, 
 * Fakulta Informanch Technologi v Brn
 * 
 * SoWindow.cpp
 *
 * SoWindow - implementation window class
 *
 * Author: Dolej Petr (xdolej06 @ stud.fit.vutbr.cz)
 *         
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

/*Soubor obsahuje nstroje pro tvorbu a sprvu okennho systmu*/

#include <stdio.h>
#include <cstdio>
#include <stdlib.h>
#include "SoWindow.h"

#define GetBit(number,bit)((1<<bit)&number)	//makro pro kontrolu masky pi inicializaci okna
											//(vrt true/false podle pozice 1cky v number)

/*kvli pedn i zprvy WM_CREATE do uivatel. funkce u child okna*/
void (*Global_Temp_ChildWndProc)(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

/*----------------------konstruktor---------------------------------------*/
SoWindow::SoWindow(){				
	UserWndProc=NULL;
	hWndChildFrame = NULL;
	hOriginalFrameMenu=NULL;
	MDIChildCounter=0;
}

/*----------------------------destruktor----------------------------------*/
SoWindow::~SoWindow(){}				

/*------------------------Inicializace a vytvoen okna-------------------*/
HWND SoWindow::Init(SoWindow_Class WindowClass,SoWindow_Parameters WindowParam){
  hInstance=GetModuleHandle(NULL);

  WNDCLASSEX wc;
	/*testovani masky a prirazovani standartniho nastaveni do struktury SoWindow_Class*/
    if (GetBit(WindowClass.mask,1) ==FALSE) {WindowClass.style=CS_HREDRAW | CS_VREDRAW;}
	if (GetBit(WindowClass.mask,2) ==FALSE) {WindowClass.lpfnWndProc=WndProc;}
	if (GetBit(WindowClass.mask,3) ==FALSE) {WindowClass.cbClsExtra=0;}
	if (GetBit(WindowClass.mask,4) ==FALSE) {WindowClass.cbWndExtra=0;}
	if (GetBit(WindowClass.mask,5) ==FALSE) {WindowClass.hInstance=hInstance;}
    if (GetBit(WindowClass.mask,6) ==FALSE) {WindowClass.Id_Icon=IDI_ASTERISK;}
	if (GetBit(WindowClass.mask,7) ==FALSE) {WindowClass.Id_SmallIcon=NULL;}
	if (GetBit(WindowClass.mask,8) ==FALSE) {WindowClass.Id_Cursor=IDC_ARROW;}
	if (GetBit(WindowClass.mask,9) ==FALSE) {WindowClass.Color_Background=COLOR_BTNSHADOW;}
	if (GetBit(WindowClass.mask,10)==FALSE) {WindowClass.lpszMenuName=NULL;}
	if (GetBit(WindowClass.mask,11)==FALSE) {WindowClass.lpszClassName=TEXT("SoWindow_CLASS");}
    
	wc.cbSize = sizeof(wc);
    wc.style = WindowClass.style;
    wc.lpfnWndProc = WindowClass.lpfnWndProc;
    wc.cbClsExtra = WindowClass.cbClsExtra;
    wc.cbWndExtra = WindowClass.cbWndExtra;
    wc.hInstance = WindowClass.hInstance;
    wc.hIcon = LoadIcon(NULL, MAKEINTRESOURCE(WindowClass.Id_Icon));
	if (WindowClass.Id_SmallIcon==0)
	   {wc.hIconSm = NULL;}
	else
	  {wc.hIconSm = LoadIcon(NULL, MAKEINTRESOURCE(WindowClass.Id_SmallIcon));}
    wc.hCursor = (HCURSOR)LoadCursor(NULL, MAKEINTRESOURCE(WindowClass.Id_Cursor));
    wc.hbrBackground = (HBRUSH)(WindowClass.Color_Background);
    wc.lpszMenuName  = WindowClass.lpszMenuName;
    wc.lpszClassName = WindowClass.lpszClassName;
    
	if(!RegisterClassEx(&wc))	/*registrace WNDCLASSEX*/
	{
	 MessageBox(NULL,"Window Registration Failed!", "Error!",MB_OK);
	 int a=GetLastError();
	 return 0;
	}

	if(!CreateNewDocument(hInstance,(char*)WindowClass.lpszClassName)){return 0;}
    
	/*testovani masky a prirazovani standartniho nastaveni do struktury SoWindow_Parameters*/
	if (GetBit(WindowParam.mask,1) ==FALSE) {WindowParam.dwExStyle=0;}
	if (GetBit(WindowParam.mask,2) ==FALSE) {WindowParam.lpClassName=TEXT("SoWindow_CLASS");}
	if (GetBit(WindowParam.mask,3) ==FALSE) {WindowParam.lpWindowName=TEXT("SoWindow");}
	if (GetBit(WindowParam.mask,4) ==FALSE) {WindowParam.dwStyle=WS_OVERLAPPEDWINDOW|WS_CLIPCHILDREN;}
	if (GetBit(WindowParam.mask,5) ==FALSE) {WindowParam.x=CW_USEDEFAULT;}
	if (GetBit(WindowParam.mask,6) ==FALSE) {WindowParam.y=CW_USEDEFAULT;}
	if (GetBit(WindowParam.mask,7) ==FALSE) {WindowParam.nWidth=CW_USEDEFAULT;}
	if (GetBit(WindowParam.mask,8) ==FALSE) {WindowParam.nHeight=CW_USEDEFAULT;}
	if (GetBit(WindowParam.mask,9) ==FALSE) {WindowParam.hWndParent=NULL;}
	if (GetBit(WindowParam.mask,10)==FALSE) {WindowParam.hMenu=NULL;}

	hWnd = CreateWindowEx(WindowParam.dwExStyle, /*vytvoen frame okna*/
		                  WindowParam.lpClassName,
						  WindowParam.lpWindowName,
						  WindowParam.dwStyle,
						  WindowParam.x,
						  WindowParam.y,
						  WindowParam.nWidth,
						  WindowParam.nHeight,
						  WindowParam.hWndParent,
						  WindowParam.hMenu,
						  GetModuleHandle(NULL),
						  this);
	if(hWnd == NULL)
	{
	MessageBox(NULL, "Window Creation Failed!", "Error!",MB_ICONEXCLAMATION | MB_OK);
	return 0;
	}
return hWnd;}

/*----------------Zobrazen okna------------------------*/
void SoWindow::Show(int iCmdShow){
	if (iCmdShow==-1){iCmdShow=SW_SHOWNORMAL;}
	ShowWindow(hWnd,iCmdShow);
    UpdateWindow(hWnd);
}

/*----------------hlavn smyka obsluhy zprv okna------------------------*/
int SoWindow::mainLoop(){ 
	MSG msg;
	while(GetMessage(&msg, NULL, 0, 0))
	{
		if (!TranslateMDISysAccel(hWndChildFrame, &msg))
		{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
		}
	}
    return msg.wParam;
}

/*------------------vrt handle okna-------------------------------------*/
HWND SoWindow::GetHWND()	
{
    return hWnd;
}

/*---------------vrt handle aktivnho mdi okna--------------------------*/
HWND SoWindow::GetMDIActiveHWND()
{
    return (HWND)SendMessage(hWndChildFrame, WM_MDIGETACTIVE,0,0);
}

/*------------------------zave okno--------------------------------------*/
BOOL SoWindow::Close()	
{
	return (::SendMessage(hWnd,WM_CLOSE,0,0));
}

/*----------------zave okno specifikovan handlem------------------------ */
BOOL SoWindow::Close(HWND hWindow)
{
	return (::SendMessage(hWindow,WM_CLOSE,0,0));
}

/*---------------nastaven max a min velikosti okna------------------------*/
BOOL SoWindow::SetMinMaxInfo(int MaxSizeX,int MaxSizeY,int MaxPositionX,int MaxPositionY,int MaxTrackSizeX,int MaxTrackSizeY,int MinTrackSizeX,int MinTrackSizeY){
	LPMINMAXINFO minmaxinfo = (tagMINMAXINFO *) malloc(sizeof(tagMINMAXINFO)); 
	  minmaxinfo->ptMaxSize.x = MaxSizeX; 
      minmaxinfo->ptMaxSize.y = MaxSizeY; 
      minmaxinfo->ptMaxPosition.x = MaxPositionX; 
      minmaxinfo->ptMaxPosition.y = MaxPositionY; 
      minmaxinfo->ptMaxTrackSize.x = MaxTrackSizeX; 
      minmaxinfo->ptMaxTrackSize.y = MaxTrackSizeY; 
      minmaxinfo->ptMinTrackSize.x = MinTrackSizeX; 
      minmaxinfo->ptMinTrackSize.y = MinTrackSizeY;
	  return(SetProp(hWnd,"USER_MINMAXINFO",(HANDLE)minmaxinfo));
};

/*-----------------nastaven max a min velikosti MDI okna-----------------*/
BOOL SoWindow::SetMDIMinMaxInfo(HWND hWindow,int MaxSizeX,int MaxSizeY,int MaxPositionX,int MaxPositionY,int MaxTrackSizeX,int MaxTrackSizeY,int MinTrackSizeX,int MinTrackSizeY){
	  LPMINMAXINFO minmaxinfo = (tagMINMAXINFO *) malloc(sizeof(tagMINMAXINFO));
	  minmaxinfo->ptMaxSize.x = MaxSizeX; 
      minmaxinfo->ptMaxSize.y = MaxSizeY; 
      minmaxinfo->ptMaxPosition.x = MaxPositionX; 
      minmaxinfo->ptMaxPosition.y = MaxPositionY; 
      minmaxinfo->ptMaxTrackSize.x = MaxTrackSizeX; 
      minmaxinfo->ptMaxTrackSize.y = MaxTrackSizeY; 
      minmaxinfo->ptMinTrackSize.x = MinTrackSizeX; 
      minmaxinfo->ptMinTrackSize.y = MinTrackSizeY;
	  return(SetProp(hWindow,"USER_MINMAXINFO",(HANDLE)minmaxinfo));
};

/*-----------------nastaven tiluku okna podle handle----------------------*/
BOOL SoWindow::SetMDIWindowTitle(HWND hWindow,LPCTSTR Title)
{
	 return(SetWindowText(hWindow,Title));
}

/*-----------------nastaven tiluku okna ---------------------------------*/
BOOL SoWindow::SetWindowTitle(LPCTSTR Title)
{
	 return(SetWindowText(hWnd,Title));
}

/*-----------------nastaven pozice okna ---------------------------------*/
BOOL SoWindow::SetWindowPos(HWND hWndInsertAfter,int X,int Y,int cx,int cy,UINT uFlags)
{
	return(::SetWindowPos(hWnd,hWndInsertAfter,X,Y,cx,cy,uFlags));
}

/*-----------------nastaven static okna ---------------------------------*/
BOOL SoWindow::SetStaticWindowPos(int X,int Y)
{
	/*nastaven statick vlastnosti okna*/
	BOOL   a= SetProp(hWnd,"USER_STATIC_X",(HANDLE)X);
	if (a) a= SetProp(hWnd,"USER_STATIC_Y",(HANDLE)Y);
	if (a) a= SetProp(hWnd,"USER_STATIC_W",(HANDLE)-1);
	if (a) a= SetProp(hWnd,"USER_STATIC_H",(HANDLE)-1);	
	if (a) a= SetProp(hWnd,"USER_STATIC_MINMAX",(HANDLE)1);
	return(a);  
}

/*-----------------nastaven static okna podle handle----------------------*/
BOOL SoWindow::SetMDIStaticWindowPos(HWND MDIhandle,int X,int Y)
{
	/*nastaven statick vlastnosti okna*/
	BOOL   a= SetProp(MDIhandle,"USER_STATIC_X",(HANDLE)X);
	if (a) a= SetProp(MDIhandle,"USER_STATIC_Y",(HANDLE)Y);
	if (a) a= SetProp(MDIhandle,"USER_STATIC_W",(HANDLE)-1);
	if (a) a= SetProp(MDIhandle,"USER_STATIC_H",(HANDLE)-1);	
	if (a) a= SetProp(MDIhandle,"USER_STATIC_MINMAX",(HANDLE)1);
	return(a);  
}

/*-----------------nastaven max a min posunu okna-------------------------*/
BOOL SoWindow::SetMinMaxWindowPos(int max_x,int max_y,int min_x,int min_y)
{
	
	BOOL   a= SetProp(hWnd,"USER_STATIC_X",(HANDLE)max_x);
	if (a) a= SetProp(hWnd,"USER_STATIC_Y",(HANDLE)max_y);
	if (a) a= SetProp(hWnd,"USER_STATIC_W",(HANDLE)min_x);
	if (a) a= SetProp(hWnd,"USER_STATIC_H",(HANDLE)min_y);	
	if (a) a= SetProp(hWnd,"USER_STATIC_MINMAX",(HANDLE)2);	
	return(a);  
}

/*----------------------nastaven max a min posunu okna--------------------*/
BOOL SoWindow::SetMDIMinMaxWindowPos(HWND MDIhandle,int max_x,int max_y,int min_x,int min_y)
{
	/*nastaven max a min posunu okna*/
	BOOL   a= SetProp(MDIhandle,"USER_STATIC_X",(HANDLE)max_x);
	if (a) a= SetProp(MDIhandle,"USER_STATIC_Y",(HANDLE)max_y);
	if (a) a= SetProp(MDIhandle,"USER_STATIC_W",(HANDLE)min_x);
	if (a) a= SetProp(MDIhandle,"USER_STATIC_H",(HANDLE)min_y);	
	if (a) a= SetProp(MDIhandle,"USER_STATIC_MINMAX",(HANDLE)2);	
	return(a);  
}

/*------------------Vytvoen novho MDI okna---------------------------*/
BOOL SoWindow::CreateNewDocument(HINSTANCE hInstance,char* ClassName)//registrace MDI
{
  WNDCLASSEX WndClsEx;
  WndClsEx.cbSize = sizeof(WNDCLASSEX);
  WndClsEx.style = CS_HREDRAW | CS_VREDRAW;
  WndClsEx.lpfnWndProc = ChildWndProc;
  WndClsEx.cbClsExtra = 0;
  WndClsEx.cbWndExtra = 0;
  WndClsEx.hInstance = hInstance;
  WndClsEx.hIcon = LoadIcon(NULL, IDI_WARNING);
  WndClsEx.hCursor = LoadCursor(NULL, IDC_ARROW);
  WndClsEx.hbrBackground = (HBRUSH)(COLOR_BTNFACE + 1);
  WndClsEx.lpszMenuName  = NULL;
  CounterWindow=CounterWindow+1;
  char ChildClassName[11]="MDICLient ";
  ChildClassName[9]=CounterWindow;
  WndClsEx.lpszClassName = ChildClassName;
  WndClsEx.hIconSm = LoadIcon(NULL, IDI_WARNING);

  if(!RegisterClassEx(&WndClsEx))
  {
   MessageBox(NULL,
        "There was a problem when attempting to create a document window",
        "Application Error",MB_OK);
   return FALSE;
  }
  else
  return TRUE;
}

/*--------------pidn standartnho menu pro prci s MDI okny do menu ----*/
void SoWindow::AddMDIFrameStandartdMenu(void){
	HMENU hMenu=GetMenu(SoWindow::hWnd);
	if (!hMenu){
		hMenu = CreateMenu();
		hOriginalFrameMenu=hMenu;
	}
    HMENU hPopUpMenu = CreateMenu();
    AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_CLOSE, "&Close");
    AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_TILE, "&Tile");
    AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_CASCADE, "C&ascade");
    AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_ICONS, "Arrange &Icons");
    AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_CLOSE_ALL, "Cl&ose All");
    AppendMenu(hMenu, MF_POPUP, (unsigned int) hPopUpMenu , "Windows");
  
    SetMenu(SoWindow::hWnd, hMenu);
    DrawMenuBar(SoWindow::hWnd);
}

/*vytvoen novho MDI okna*/
HWND SoWindow::CreateNewMDIChild(char szTitle[],BOOL AddChildMenu,HMENU hChildMenu,void (*ChildWndProc)(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam),int style)
{ MDIChildCounter++;
  if (MDIChildCounter==1){
	
	HMENU hMenu=GetMenu(hWnd);			// uloen originlnho handle menu frame okna
	if (hOriginalFrameMenu==NULL)
		hOriginalFrameMenu=hMenu;
    
	  
	  CLIENTCREATESTRUCT ccs;	
	  ccs.hWindowMenu  = NULL;
	  ccs.idFirstChild = StartChildrenNo;

	  hWndChildFrame = CreateWindowEx(WS_EX_CLIENTEDGE, /*vytvoen framu v okne okna*/
		  					    	  "MDICLIENT",
		  							  NULL,
									  WS_CHILD | WS_CLIPCHILDREN | WS_VSCROLL
									  | WS_HSCROLL | WS_VISIBLE,
									  CW_USEDEFAULT,
									  CW_USEDEFAULT,
									  680,
									  460,
									  hWnd,
								 	  NULL,
									  GetModuleHandle(NULL),
									  (LPVOID)&ccs);

	  if(hWndChildFrame == NULL){
		  MessageBox(hWnd, "Could not create MDI client.", "Error", MB_OK | MB_ICONERROR);
	  }
    }
  MDICREATESTRUCT mcs;
  HWND NewWnd;
  mcs.szTitle = szTitle;
  char ChildClassName[11]="MDICLient ";
  ChildClassName[9]=CounterWindow;
  mcs.szClass =   ChildClassName;
  mcs.hOwner  = GetModuleHandle(NULL);
  mcs.x = mcs.cx = CW_USEDEFAULT;
  mcs.y = mcs.cy = CW_USEDEFAULT;
  mcs.style = style;

  Global_Temp_ChildWndProc=NULL;
  if (ChildWndProc)
	{Global_Temp_ChildWndProc=ChildWndProc;}
  NewWnd = (HWND)SendMessage(hWndChildFrame, WM_MDICREATE, 0, (LONG)&mcs);

if( !NewWnd )
{
  MessageBox(NULL,
			"Error creating child window",
			"Creation Error",
			MB_OK);}


 if(!SetProp(NewWnd,"USER_FUNCTION",ChildWndProc)){  /*nastaven vlastn obsluhy zprv child okna*/
    MessageBox(NULL,
	  	       "Error set properities (USER_FUNCTION) to child window",
			   "Creation properities Error",
			   MB_OK);
  }
 
 if(!SetProp(NewWnd,"USER_CHILD_MENU",hChildMenu)){  /*nastaven specilnho menu child okna*/
    MessageBox(NULL,
	  	       "Error set properities (USER_CHILD_MENU) to child window",
			   "Creation properities Error",
			   MB_OK);
  }

 if(!SetProp(NewWnd,"USER_CHILD_BMENU",(HANDLE)AddChildMenu)){  /*nastaven zda je pidno "windows"*/
    MessageBox(NULL,
	  	       "Error set properities (USER_CHILD_BMENU) to child window",
			   "Creation properities Error",
			   MB_OK);
  }
  
  if(!SetProp(hWnd,"USER_MENU_PROCES",NULL)){  /*nastaven voln specilnho menu child okna*/
      MessageBox(NULL,
	  "Error set properities (USER_MENU_PROCES) to child window",
	  "Creation properities Error",
	  MB_OK);
  }
return NewWnd;
}



/*--------------------------obsluha zprv hlavnho okna---------------------*/
LONG CALLBACK SoWindow::WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{   
	SoWindow * pSoWindow = (SoWindow *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
	//http://www.windev.cz/?kod=102&obsah=2003/9/vc_wndproc_v_tride	
		switch(uMsg)
		{   
		case WM_CREATE:{
            SetWindowLongPtr(hWnd, GWLP_USERDATA, (LONG_PTR)((LPCREATESTRUCT)lParam)->lpCreateParams);
			SoWindow * pSoWindow = (SoWindow *)GetWindowLongPtr(hWnd, GWLP_USERDATA);
			if (*(pSoWindow->UserWndProc)!=NULL) 
				pSoWindow->UserWndProc(hWnd,uMsg,wParam,lParam);
			::SetWindowPos(hWnd,HWND_BOTTOM,0,0,800,600,SWP_SHOWWINDOW|SWP_NOSIZE|SWP_NOMOVE);
			break;}
		case WM_GETMINMAXINFO:
			{
				  LPMINMAXINFO minmaxinfo = (LPMINMAXINFO) GetProp(hWnd,"USER_MINMAXINFO");				
				  if (minmaxinfo)
				  {
				    LPMINMAXINFO minmax;
				    minmax = (LPMINMAXINFO)lParam;
				    if (minmaxinfo->ptMaxPosition.x>0)  minmax->ptMaxPosition.x  = minmaxinfo->ptMaxPosition.x;
				    if (minmaxinfo->ptMaxPosition.y>0)  minmax->ptMaxPosition.y  = minmaxinfo->ptMaxPosition.y;
				    if (minmaxinfo->ptMaxSize.x>0)		minmax->ptMaxSize.x	     = minmaxinfo->ptMaxSize.x;
				    if (minmaxinfo->ptMaxSize.y>0)		minmax->ptMaxSize.y	     = minmaxinfo->ptMaxSize.y;
				    if (minmaxinfo->ptMaxTrackSize.x>0) minmax->ptMaxTrackSize.x = minmaxinfo->ptMaxTrackSize.x;
				    if (minmaxinfo->ptMaxTrackSize.y>0) minmax->ptMaxTrackSize.y = minmaxinfo->ptMaxTrackSize.y;
				    if (minmaxinfo->ptMinTrackSize.x>0) minmax->ptMinTrackSize.x = minmaxinfo->ptMinTrackSize.x;
				    if (minmaxinfo->ptMinTrackSize.y>0) minmax->ptMinTrackSize.y = minmaxinfo->ptMinTrackSize.y;
				  }
			
			break;
			}
		case WM_MOVING:{
	/*pi pohybu s oknem se zjist zda nebylo nastaveno njak omezen okna a ppadn se provede*/
			RECT * pRect=(RECT *)lParam;
			int delka=pRect->right-pRect->left;
			int vyska=pRect->bottom-pRect->top;

			int STATICKE = (int) GetProp(hWnd,"USER_STATIC_MINMAX");
			int max_y = (int) GetProp(hWnd,"USER_STATIC_Y");
			int max_x = (int) GetProp(hWnd,"USER_STATIC_X");
			int min_x = (int) GetProp(hWnd,"USER_STATIC_W");
			int	min_y = (int) GetProp(hWnd,"USER_STATIC_H");
		
			if (STATICKE!=0){
		
			if (STATICKE==1){
	
				if (max_x>=0||max_y>=0)
				{
				pRect->left=max_x;
				pRect->right=pRect->left+delka;
				}
				if (max_y>=0||max_x>=0) 
				{
				pRect->top=max_y;
				pRect->bottom=pRect->top+vyska;
				}
			}
			else{
			
				if (max_x>=0)
				if (pRect->right>max_x+delka)
				{
				pRect->right=max_x+delka;
				pRect->left=pRect->right-delka;
				}
				if (min_x>=0)
				if (pRect->left<min_x)
				{
				pRect->left=min_x;
				pRect->right=pRect->left+delka;
					}
				if (min_y>=0)
				if (pRect->top<min_y)
				 {
				pRect->top=min_y;
				pRect->bottom=pRect->top+vyska;
				}
				if (max_y>=0) 
				if (pRect->bottom>max_y+vyska)
				 {
				 pRect->bottom=max_y+vyska;
				 pRect->top=pRect->bottom-vyska;
				 }
				}
			}

		break;}
		case WM_NCDESTROY:
		{/*odstrann properities okna z pamti*/
			RemoveProp(hWnd,"USER_FUNCTION");					
			RemoveProp(hWnd,"USER_CHILD_MENU");
			RemoveProp(hWnd,"USER_CHILD_BMENU");
			RemoveProp(hWnd,"USER_MENU_PROCES");
			RemoveProp(hWnd,"USER_STATIC_X");					
			RemoveProp(hWnd,"USER_STATIC_Y");
			RemoveProp(hWnd,"USER_STATIC_H");
			RemoveProp(hWnd,"USER_STATIC_MINMAX");
			RemoveProp(hWnd,"USER_MINMAXINFO");
		}
		break;
        case WM_DESTROY:
            PostQuitMessage(0);
            return 0;
		case MENU_MSG:
			{
			 void (*CALL)(int);
				 CALL = (void (*)(int)) GetProp(HWND(lParam),"USER_CALL_MENU");
				 if (CALL)
				 {
					CALL((int)wParam); /*volej uzivatelovu funkci*/
				 }

			}
		case WM_COMMAND:{
		if (pSoWindow){
		  HWND hChild = (HWND)SendMessage(pSoWindow->hWndChildFrame, WM_MDIGETACTIVE,0,0);
		  void (*CALL)(int);
		  CALL = (void (*)(int)) GetProp(hChild,"USER_MENU_PROCES"); /*zskn uivatelsk funkce menu*/
		  if (CALL) CALL(LOWORD(wParam));
		}
		switch(LOWORD(wParam))
		  {/*reace na vbr nkter z poloky menu Windows*/
			case IDM_WINDOW_CLOSE:
				if (pSoWindow)
				 {
				  HWND hChild = (HWND)SendMessage(pSoWindow->hWndChildFrame, WM_MDIGETACTIVE,0,0);
				  if(hChild)
				    {SendMessage(hChild, WM_CLOSE, 0, 0);}
				 }
			break;
			case IDM_WINDOW_TILE:
				  if (pSoWindow)
				    SendMessage(pSoWindow->hWndChildFrame, WM_MDITILE, 0, 0);
			break;
			case IDM_WINDOW_CASCADE:
				  if (pSoWindow)
				    SendMessage(pSoWindow->hWndChildFrame, WM_MDICASCADE, 0, 0);
			break;
			case IDM_WINDOW_ICONS:
				  if (pSoWindow)
				    SendMessage(pSoWindow->hWndChildFrame, WM_MDIICONARRANGE, 0, 0);
			break;
			case IDM_WINDOW_CLOSE_ALL:
				 if (pSoWindow)
				  {
					HWND hWndCurrent;
					do {
						hWndCurrent = (HWND)SendMessage(pSoWindow->hWndChildFrame, WM_MDIGETACTIVE,0,0);
						SendMessage(hWndCurrent, WM_CLOSE, 0, 0);
					}while(hWndCurrent);
				  }
			break;
		   default:
		   {
			if(LOWORD(wParam) >= StartChildrenNo)
			  if (pSoWindow)
			   {DefFrameProc(hWnd, pSoWindow->hWndChildFrame, WM_COMMAND, wParam, lParam);}
			else 
			{
			  if (pSoWindow){ 
			     HWND hWndCurrent = (HWND)SendMessage(pSoWindow->hWndChildFrame, WM_MDIGETACTIVE,0,0);
			       if( hWndCurrent )
			        {SendMessage(hWndCurrent, WM_COMMAND, wParam, lParam);}
			  }
			}
		   }
		  }
						}
		  break;
	}//switch
    if (pSoWindow)
	/*pokud si uzivatel preje vlastni obsluhu*/
    if (*(pSoWindow->UserWndProc)!=NULL) 
		pSoWindow->UserWndProc(hWnd,uMsg,wParam,lParam);
    
	if (pSoWindow)
	  {return DefFrameProc(hWnd, pSoWindow->hWndChildFrame, uMsg, wParam, lParam);}
	else {return DefFrameProc(hWnd, NULL, uMsg, wParam, lParam);}

}



/*----------------------obsluha zprv child oken---------------------------*/
LONG CALLBACK SoWindow::ChildWndProc(HWND hwndChild, UINT msg, WPARAM wParam, LPARAM lParam)
{   
	void (*ChildWndProc)(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
	ChildWndProc =	(void (*)(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)) GetProp(hwndChild,"USER_FUNCTION"); /*zskn uivatelsk funkce*/
	HWND hWndMDIClient = GetParent(hwndChild);
    HWND hWndMainFrame = GetParent(hWndMDIClient);
	SoWindow * pSoWindow = (SoWindow *)GetWindowLongPtr(hWndMainFrame, GWLP_USERDATA);
	
switch(msg)
{  
  case WM_CREATE:{
	  if (Global_Temp_ChildWndProc){ /*pokud si uzivatel preje vlastni obsluhu zprv*/
		  Global_Temp_ChildWndProc(hwndChild,msg,wParam,lParam);}
	}
  case WM_NCDESTROY:{/*odstrann properities okna z pamti*/
	 RemoveProp(hwndChild,"USER_FUNCTION");					
	 RemoveProp(hwndChild,"USER_CHILD_MENU");
	 RemoveProp(hwndChild,"USER_CHILD_BMENU");
	 RemoveProp(hwndChild,"USER_MENU_PROCES");
	 RemoveProp(hwndChild,"USER_STATIC_X");					
	 RemoveProp(hwndChild,"USER_STATIC_Y");
	 RemoveProp(hwndChild,"USER_STATIC_H");
	 RemoveProp(hwndChild,"USER_STATIC_MINMAX");
	 RemoveProp(hwndChild,"USER_MINMAXINFO");
  }
  break;

  case WM_MOVING:{ 
  /*pi pohybu s oknem se zjist zda nebylo nastaveno njak omezen okna a ppadn se provede*/
		RECT * pRect=(RECT *)lParam;
		int delka=pRect->right-pRect->left;
		int vyska=pRect->bottom-pRect->top;
		RECT * pClientRect=(RECT *)malloc(sizeof(RECT));
		GetWindowRect(hWndMDIClient,pClientRect);

		int STATICKE = (int) GetProp(hwndChild,"USER_STATIC_MINMAX");
		int max_y = (int) GetProp(hwndChild,"USER_STATIC_Y");
		int max_x = (int) GetProp(hwndChild,"USER_STATIC_X");
		int min_x = (int) GetProp(hwndChild,"USER_STATIC_W");
		int	min_y = (int) GetProp(hwndChild,"USER_STATIC_H");
		
		if (STATICKE!=0){
		
		if (STATICKE==1)
		{
	
			if (max_x>=0||max_y>=0)
			  {
			  pRect->left=max_x+pClientRect->left;
			  pRect->right=pRect->left+delka;
			  }
			if (max_y>=0||max_x>=0) 
			  {
			  pRect->top=max_y+pClientRect->top;
			  pRect->bottom=pRect->top+vyska;
			  }
		}
		else{
			
			if (max_x>=0)
			if (pRect->right>max_x+pClientRect->left+delka)
			  {
			  pRect->right=max_x+pClientRect->left+delka;
			  pRect->left=pRect->right-delka;
			   }
            if (min_x>=0)
			 if (pRect->left<min_x+pClientRect->left)
			  {
			  pRect->left=min_x+pClientRect->left;
			  pRect->right=pRect->left+delka;
				}
			 if (min_y>=0)
			  if (pRect->top<min_y+pClientRect->top)
			  {
			  pRect->top=min_y+pClientRect->top;
			  pRect->bottom=pRect->top+vyska;
			  }
			 if (max_y>=0) 
			  if (pRect->bottom>max_y+pClientRect->top+vyska)
			  {
			  pRect->bottom=max_y+pClientRect->top+vyska;
			  pRect->top=pRect->bottom-vyska;
			  }
			}
		}

  break;}

  case WM_MDIACTIVATE:{
	  /*pi aktviaci MDI okna se provede zmna menu ve frame okn*/
   if (pSoWindow){
	HMENU hMenu;
	UINT EnableFlag;
	hMenu = pSoWindow->hOriginalFrameMenu;
	  
	if (hMenu){	
	if(hwndChild == (HWND)lParam)
	  {EnableFlag = MF_ENABLED;}
	else
	  {EnableFlag = MF_GRAYED;}

	EnableMenuItem(hMenu, IDM_WINDOW_CLOSE, MF_BYCOMMAND| EnableFlag);
	EnableMenuItem(hMenu, IDM_WINDOW_TILE, MF_BYCOMMAND | EnableFlag);
	EnableMenuItem(hMenu, IDM_WINDOW_CASCADE, MF_BYCOMMAND| EnableFlag);
	EnableMenuItem(hMenu, IDM_WINDOW_TILE, MF_BYCOMMAND | EnableFlag);
	EnableMenuItem(hMenu, IDM_WINDOW_ICONS, MF_BYCOMMAND| EnableFlag);
	EnableMenuItem(hMenu, IDM_WINDOW_CLOSE_ALL, MF_BYCOMMAND | EnableFlag);
	DrawMenuBar(hWndMainFrame);
	}
	int a=0;
	char buffer[8]="\0";

	HMENU hChildMenu = (HMENU) GetProp(hwndChild,"USER_CHILD_MENU"); /*zskn uivatelsk funkce*/
	BOOL AddChildMenu = (BOOL) GetProp(hwndChild,"USER_CHILD_BMENU");  //pokud m bt pidvno menu "windows"
	
	if ((hChildMenu==NULL)&&(AddChildMenu==TRUE))	// pokud nechce vlastn menu ale chce aspon "windows"
		{
		 HMENU hNewMenu = CreateMenu();
		 HMENU hPopUpMenu = CreateMenu();
		 AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_CLOSE, "&Close");
		 AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_TILE, "&Tile");
		 AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_CASCADE, "C&ascade");
		 AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_ICONS, "Arrange &Icons");
		 AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_CLOSE_ALL, "Cl&ose All");
		 AppendMenu(hNewMenu, MF_POPUP, (unsigned int) hPopUpMenu , "Windows");
		 hMenu=hNewMenu;
		  if(!SetProp(hwndChild,"USER_CHILD_MENU",(HANDLE)hMenu)){  /*nastaven zda je pidno "windows"*/
				MessageBox(NULL,
	  	       "Error set properities (USER_CHILD_BMENU) to child window",
			   "Creation properities Error",
			   MB_OK);
			}
		}

	if ((hwndChild==(HWND)lParam)&&(hChildMenu))
		{   //nastaven novho menu
			
			if (AddChildMenu){ 

				do{
					GetMenuString(hChildMenu,a,buffer,8, MF_BYPOSITION);
	    			a++;
				}while ((strcmp(buffer,"Windows")!=0)&&(a!=20));
				
				if (a==20){
				  HMENU hPopUpMenu = CreateMenu();
				  AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_CLOSE, "&Close");
				  AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_TILE, "&Tile");
				  AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_CASCADE, "C&ascade");
				  AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_ICONS, "Arrange &Icons");
				  AppendMenu(hPopUpMenu, MF_STRING, IDM_WINDOW_CLOSE_ALL, "Cl&ose All");
				  AppendMenu(hChildMenu, MF_POPUP, (unsigned int) hPopUpMenu , "Windows");
				}
			}
			a=0;
			do{
				GetMenuString(hChildMenu,a,buffer,8, MF_BYPOSITION);
	    		a++;
			}while ((strcmp(buffer,"Windows")!=0)&&(a!=20));
			if (a==20) a=1;
			HMENU hChildSubMenu = GetSubMenu(hChildMenu,a-1);
			SendMessage(hWndMDIClient,WM_MDISETMENU,(WPARAM)hChildMenu,(LPARAM)hChildSubMenu); 
		}
		else
			//vrcen starho menu
		if (hMenu)
		{	a=0;
			do{
				GetMenuString(hMenu,a,buffer,8, MF_BYPOSITION);
	    		a++;
			}while ((strcmp(buffer,"Windows")!=0)&&(a!=20));
			if (a==20) a=0;
			HMENU hMainSubMenu = GetSubMenu(hMenu,a-1);    
			SendMessage(hWndMDIClient,WM_MDISETMENU,(WPARAM)hMenu,(LPARAM)hMainSubMenu); 
		}else
		{SetMenu(hWndMainFrame, NULL);}	//pokud neni zadan menu

	DrawMenuBar(hWndMainFrame);
  }//{if (pSoWindow)}
  }
  break;
  
  case WM_GETMINMAXINFO:
    {
		LPMINMAXINFO minmaxinfo = (LPMINMAXINFO) GetProp(hwndChild,"USER_MINMAXINFO");				
		if (minmaxinfo)
		  {
		   LPMINMAXINFO minmax;
		   minmax = (LPMINMAXINFO)lParam;
		   if (minmaxinfo->ptMaxPosition.x>0)  minmax->ptMaxPosition.x  = minmaxinfo->ptMaxPosition.x;
		   if (minmaxinfo->ptMaxPosition.y>0)  minmax->ptMaxPosition.y  = minmaxinfo->ptMaxPosition.y;
	       if (minmaxinfo->ptMaxSize.x>0)	   minmax->ptMaxSize.x	    = minmaxinfo->ptMaxSize.x;
		   if (minmaxinfo->ptMaxSize.y>0)	   minmax->ptMaxSize.y	    = minmaxinfo->ptMaxSize.y;
		   if (minmaxinfo->ptMaxTrackSize.x>0) minmax->ptMaxTrackSize.x = minmaxinfo->ptMaxTrackSize.x;
		   if (minmaxinfo->ptMaxTrackSize.y>0) minmax->ptMaxTrackSize.y = minmaxinfo->ptMaxTrackSize.y;
		   if (minmaxinfo->ptMinTrackSize.x>0) minmax->ptMinTrackSize.x = minmaxinfo->ptMinTrackSize.x;
		   if (minmaxinfo->ptMinTrackSize.y>0) minmax->ptMinTrackSize.y = minmaxinfo->ptMinTrackSize.y;
		 }
			
	break;
    }
  default:
	  
	  if (ChildWndProc){ /*pokud si uzivatel preje vlastni obsluhu*/
		  ChildWndProc(hwndChild,msg,wParam,lParam);}

    return DefMDIChildProc(hwndChild, msg, wParam, lParam);
}
return 0;
}



/*--------------------Stadardn open dialog--------------------------------*/
BOOL Open_Dialog(LPTSTR lpFile, HWND hwndOwner,const char * files)
{
  OPENFILENAME ofn;
  TCHAR chFile[_MAX_PATH];
  lstrcpy(chFile, "");
  ZeroMemory(&ofn, sizeof(OPENFILENAME));
  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.lpstrFile = chFile;
  ofn.hwndOwner = hwndOwner;
  ofn.nMaxFile = sizeof(chFile);
  ofn.lpstrFilter = files;
  ofn.nFilterIndex = 1;
  ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;
  if ( !GetOpenFileName(&ofn) )
    return FALSE;
  lstrcpy(lpFile, chFile);
  return TRUE;
}

/*--------------------nastaven ukazatele myi na BUSSY--------------------*/
void setBusyCursor(BOOL showBusy)
{      
	if (showBusy)
	  SetCursor(LoadCursor(NULL,MAKEINTRESOURCE(IDC_APPSTARTING)));
	else
	  SetCursor(LoadCursor(NULL,MAKEINTRESOURCE(IDC_ARROW)));
	
  ShowCursor(TRUE);
}

/*--------------------Stadardn color dialog--------------------------------*/
BOOL Open_ColorDialog(DWORD * col, HWND hwndOwner)
{
CHOOSECOLOR cc;                 // common dialog box structure 
static COLORREF acrCustClr[16]; // array of custom colors 
static DWORD rgbCurrent;        // initial color selection

// Initialize CHOOSECOLOR 
ZeroMemory(&cc, sizeof(cc));
cc.lStructSize = sizeof(cc);
cc.hwndOwner = hwndOwner;
cc.lpCustColors = (LPDWORD) acrCustClr;
cc.rgbResult = rgbCurrent;
cc.Flags = CC_FULLOPEN | CC_RGBINIT;

if (ChooseColor(&cc)==TRUE) {
    //hbrush = CreateSolidBrush(cc.rgbResult);
	*col=cc.rgbResult;
	return TRUE;}
return FALSE;}

/*--------------------Oteen dokumentu dle zadan cesty--------------------------------*/
void OpenDokument(const char * FullPath)
{
  ShellExecute(NULL, "open", FullPath, NULL, NULL, SW_SHOWNORMAL);
}

/*--------------------Stadardn save dialog--------------------------------*/
BOOL Save_Dialog(LPTSTR lpFile, HWND hwndOwner,const char * files)
{
  OPENFILENAME ofn;
  TCHAR chFile[_MAX_PATH];
  lstrcpy(chFile, "");
  ZeroMemory(&ofn, sizeof(OPENFILENAME));
  ofn.lStructSize = sizeof(OPENFILENAME);
  ofn.lpstrFile = chFile;
  ofn.hwndOwner = hwndOwner;
  ofn.nMaxFile = sizeof(chFile);
  ofn.lpstrFilter = files;
  ofn.nFilterIndex = 1;
  ofn.Flags = OFN_OVERWRITEPROMPT | OFN_HIDEREADONLY;
  if ( !GetSaveFileName(&ofn) )
    return FALSE;
  lstrcpy(lpFile, chFile);
  return TRUE;
}