/*****************************************************************************\
 * Vvoj okennho systmu ve WinAPI
 * Bakalsk prce 2006, 
 * Fakulta Informanch Technologi v Brn
 * 
 * SoWindow.h
 *
 * SoWindow - window class
 *
 * Author: Dolej Petr (xdolej06 @ stud.fit.vutbr.cz)
 *         
 * Contributors:
 *
 * ----------------------------------------------------------------------------
 *
 * THIS SOFTWARE IS NOT COPYRIGHTED
 *
 * This source code is offered for use in the public domain.
 * You may use, modify or distribute it freely.
 *
 * This source code is distributed in the hope that it will be useful but
 * WITHOUT ANY WARRANTY.  ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 * DISCLAIMED.  This includes but is not limited to warranties of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * If you find the source code useful, authors will kindly welcome
 * if you give them credit and keep their names with their source code,
 * but do not feel to be forced to do so.
 *
\*****************************************************************************/

#include <windows.h> 
#ifndef _SoWIN32_WINDOW
#define _SoWIN32_WINDOW
#define MENU_MSG   WM_USER + 1000

/* SoWindow_Class MASK PARAMETERS */
#define WC_STYLE			2
#define WC_LPFNWNDPROC		4
#define WC_CBCLSEXTRA		8	
#define WC_CBWNDEXTRA		16
#define WC_HINSTANECE		32
#define WC_ID_ICON			64
#define WC_ID_SMALLICON		128 
#define WC_ID_CURSOR		256
#define WC_COLOR_BACKGROUNG 512
#define WC_LPSZMENUNAME		1024
#define WC_LPSZCLASSNAME    2048

/* SoWindow_Parameters MASK PARAMETERS */
#define WP_DWEXSTYLE	2
#define WP_LPCLASSNAME	4
#define WP_LPWINDOWNAME	8
#define WP_DWSTYLE		16
#define WP_X			32
#define WP_Y			64
#define WP_NWIDTH		128
#define WP_NHEIGHT		256
#define WP_HWNDPARENT	512
#define WP_HMENU		1024
        
#define IDM_WINDOW			 5000
#define IDM_WINDOW_CLOSE     5001  //"Closes the current document"
#define IDM_WINDOW_TILE      5002  // "Arranges child windows in a tile format"
#define IDM_WINDOW_CASCADE   5003  // "Arranges child windows in a cascade format"
#define IDM_WINDOW_ICONS     5004  // "Arranges the icons of minimized child windows"
#define IDM_WINDOW_CLOSE_ALL 5005  // "Closes all child windows"

typedef struct SoWindow_Parameters{
					DWORD mask;
					DWORD dwExStyle;
					LPCTSTR lpClassName;
					LPCTSTR lpWindowName;
					DWORD dwStyle;
					int x;
					int y;
					int nWidth;
					int nHeight;
					HWND hWndParent;
					HMENU hMenu;
}SoWindow_Parameters; 

typedef struct SoWindow_Class{
			   DWORD mask;			   
			   UINT style;
			   WNDPROC lpfnWndProc;
			   int cbClsExtra;
               int cbWndExtra;
			   HINSTANCE hInstance;
			   LPSTR Id_Icon;
			   LPSTR Id_SmallIcon;
			   LPSTR Id_Cursor;
			   int Color_Background;
			   LPCTSTR lpszMenuName;
			   LPCTSTR lpszClassName;
}SoWindow_Class;
	    

const int StartChildrenNo = 994;
static char CounterWindow='0';

class SoWindow
{
  public: 
		 SoWindow();
		~SoWindow();
	    HWND Init(SoWindow_Class,SoWindow_Parameters);
		void Show(int);
		int mainLoop();
	    void (*UserWndProc)(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
		BOOL SetMinMaxInfo(int MaxSizeX=-1,int MaxSizeY=-1,int MaxPositionX=-1,int MaxPositionY=-1,int MaxTrackSizeX=-1,int MaxTrackSizeY=-1,int MinTrackSizeX=-1,int MinTrackSizeY=-1);
		BOOL SetMDIMinMaxInfo(HWND hWindow,int MaxSizeX=-1,int MaxSizeY=-1,int MaxPositionX=-1,int MaxPositionY=-1,int MaxTrackSizeX=-1,int MaxTrackSizeY=-1,int MinTrackSizeX=-1,int MinTrackSizeY=-1);
		HWND CreateNewMDIChild(char szTitle[]="Untitled",BOOL AddChildMenu=TRUE,HMENU hChildMenu=NULL,void (*ChildWndProc)(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)=NULL,int style=MDIS_ALLCHILDSTYLES);
		void AddMDIFrameStandartdMenu(void);
		BOOL SetMDIWindowTitle(HWND hWindow,LPCTSTR Title);
		BOOL SetWindowTitle(LPCTSTR Title);
		BOOL SetWindowPos(HWND hWndInsertAfter,int X,int Y,int cx,int cy,UINT uFlags);
		BOOL SetMDIWindowPos(HWND hWindow,HWND hWndInsertAfter,int X,int Y,int cx,int cy,UINT uFlags);
		BOOL SetMinMaxWindowPos(int max_x=-1,int max_y=-1,int min_x=-1,int min_y=-1);
		BOOL SetMDIMinMaxWindowPos(HWND MDIhandle=NULL,int max_x=-1,int max_y=-1,int min_x=-1,int min_y=-1);
        BOOL SetStaticWindowPos(int X=-1,int Y=-1);
		BOOL SetMDIStaticWindowPos(HWND MDIhandle=NULL,int X=-1,int Y=-1);
		HWND GetHWND();
		HWND GetMDIActiveHWND();
		BOOL Close();
		BOOL Close(HWND hWindow);
  private:
	    HWND hWnd;
		HINSTANCE hInstance;
		HWND hWndChildFrame;
		HMENU hOriginalFrameMenu;
		int MDIChildCounter;
		int CreateNewDocument(HINSTANCE,char* ClassName);

  protected:
   static LONG CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
   static LONG CALLBACK ChildWndProc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);

};

BOOL Open_Dialog(LPTSTR lpFile, HWND hwndOwner,const char * files);
BOOL Save_Dialog(LPTSTR lpFile, HWND hwndOwner,const char * files);
BOOL Open_ColorDialog(DWORD * col, HWND hwndOwner);
void setBusyCursor(BOOL showBusy);
void OpenDokument(const char * FullPath);

#endif